@@ -2,8 +2,9 @@
  *
  * donkeyprotocol.cpp
  *
- * Copyright (C) 2003 Petter E. Stokke <gibreel@kmldonkey.org>
+ * Copyright (C) 2003 Petter Stokke <ummo@hellokitty.com>
  * Copyright (C) 2003 Sebastian Sauer <mail@dipe.org>
+ * Copyright (C) 2006 Christian Muehlhaeuser <chris@chris.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -36,6 +37,8 @@
 #include "searchquery.h"
 
 #include <qregexp.h>
+//Added by qt3to4:
+#include <Q3ValueList>
 #include <kdebug.h>
 
 DonkeyProtocol::DonkeyProtocol(bool poll, QObject *parent)
@@ -90,14 +93,18 @@ const QString& DonkeyProtocol::password()
 
 bool DonkeyProtocol::isConnected()
 {
-    return (sock.state() == QSocket::Connected);
+    return (sock.state() == Q3Socket::Connected);
 }
 
 bool DonkeyProtocol::connectToCore()
 {
     donkeyError = NoError;
     if (isConnected()) {
-        if (! disconnectFromCore()) return false;
+        kDebug() << "Is already connected. Trying to disconnect...";
+        if (! disconnectFromCore()) {
+            kWarning() << "Failed to disconnect.";
+            return false;
+        }
     }
     else
         flushState();
@@ -124,8 +131,8 @@ bool DonkeyProtocol::disconnectFromCore()
     sock.flush();
 
     sock.close();
-    kdDebug() << "Socket closed." << endl;
-    if (sock.state() == QSocket::Idle)
+    kDebug() << "Socket closed.";
+    if (sock.state() == Q3Socket::Idle) //QAbstractSocket::UnconnectedState)
         emit signalDisconnected(donkeyError);
     flushState();
     return true;
@@ -161,14 +168,15 @@ void DonkeyProtocol::socketDisconnected()
 
 void DonkeyProtocol::socketError(int err)
 {
+    kDebug() << "Socket error:" << err;
     switch (err) {
-    case QSocket::ErrConnectionRefused:
+    case Q3Socket::ErrConnectionRefused:
         emit signalDisconnected(ConnectionRefusedError);
         break;
-    case QSocket::ErrHostNotFound:
+    case Q3Socket::ErrHostNotFound:
         emit signalDisconnected(HostNotFoundError);
         break;
-    case QSocket::ErrSocketRead:
+    case Q3Socket::ErrSocketRead:
         donkeyError = CommunicationError;
         disconnectFromCore();
         break;
@@ -177,7 +185,7 @@ void DonkeyProtocol::socketError(int err)
 
 void DonkeyProtocol::pruneClientRecord(int clientno)
 {
-    QIntDictIterator<FileInfo> it(download);
+    Q3IntDictIterator<FileInfo> it(download);
     for (; it.current(); ++it) {
         it.current()->removeSource(clientno);
         emit fileUpdated(it.current()->fileNo());
@@ -187,7 +195,7 @@ void DonkeyProtocol::pruneClientRecord(int clientno)
         emit friendRemoved(clientno);
 }
 
-const int DonkeyProtocol::protocolVersion()
+const int& DonkeyProtocol::protocolVersion()
 {
     return proto;
 }
@@ -196,6 +204,7 @@ void DonkeyProtocol::processMessage()
 {
     DonkeyMessage* msg;
     while ((msg = sock.popMessage())) {
+        //kDebug()<<"DonkeyProtocol::processMessage opcode="<<msg->opcode();
         DonkeyMessage* out;
         QString baz;
         emit messageReceived(msg);
@@ -204,9 +213,9 @@ void DonkeyProtocol::processMessage()
             proto = msg->readInt32();
             // FIXME: We might read the protocol 26 max_to_gui and max_from_gui values here (both int32).
             // We don't, because we don't really need them for anything useful right now.
-            kdDebug() << "CoreProtocol message, version " << proto << endl;
+            kDebug()<<"CoreProtocol message, version="<<proto<<"MIN_PROTOCOL_VERSION="<<MIN_PROTOCOL_VERSION<<"MAX_PROTOCOL_VERSION="<<MAX_PROTOCOL_VERSION;
             if (proto < MIN_PROTOCOL_VERSION) {
-                kdDebug() << "Obsolete protocol version!" << endl;
+                kWarning() << "Obsolete protocol version!";
                 donkeyError = IncompatibleProtocolError;
                 disconnectFromCore();
                 break;
@@ -217,7 +226,7 @@ void DonkeyProtocol::processMessage()
             coreProto = proto;
             if (proto > MAX_PROTOCOL_VERSION)
                 proto = MAX_PROTOCOL_VERSION;
-            kdDebug() << "Using protocol " << proto << endl;
+            kDebug()<<"Using protocol="<<proto<<"wantpoll="<<wantpoll;
             delete out;
             if (wantpoll) {
                 out = new DonkeyMessage(GuiExtensions);
@@ -232,6 +241,7 @@ void DonkeyProtocol::processMessage()
             out->writeString(uname);
             sock.sendMessage(*out);
             delete out;
+            if (proto >= 31) requestVersion();
             emit signalConnected();
             break;
         case Console: {
@@ -242,10 +252,10 @@ void DonkeyProtocol::processMessage()
                 QRegExp rx("^\n[\\-]+\nEval[\\s]command:[\\s](.+)");
                 if (rx.search(baz) >= 0) {
                     QString s = rx.cap(1);
-                    int pos = s.find("\n");
+                    const int pos = s.find("\n");
                     if (pos > 0) {
-                        QString cmd = s.left(pos).stripWhiteSpace();
-                        QString res = s.mid(pos + 2, s.length() - pos - 4).stripWhiteSpace();
+                        const QString cmd = s.left(pos).trimmed();
+                        const QString res = s.mid(pos + 2, s.length() - pos - 4).trimmed();
                         ConsoleCallbackInterface* cb = consoleCallbacks[cmd];
                         if (cb && !res.startsWith("No such command ")) {
                             consoleCallbacks.remove(cmd);
@@ -259,14 +269,14 @@ void DonkeyProtocol::processMessage()
             if (! ok) emit consoleMessage(baz);
         } break;
         case BadPassword:
-            kdDebug() << "Bad password!" << endl;
+            kDebug() << "Bad password!";
             donkeyError = AuthenticationError;
             disconnectFromCore();
             break;
         case Client_stats_v1:
         case Client_stats_v2:
         case Client_stats_v3:
-            kdDebug() << "Obsolete client stats message received" << endl;
+            kDebug() << "Obsolete client stats message received";
             break;
         case Client_stats:
         {
@@ -406,13 +416,13 @@ void DonkeyProtocol::processMessage()
         /*
         case Client_file:
         {
-            kdDebug() << "Client_file message: " << msg->readInt32() << " \"" << msg->readString() << "\" " << msg->readInt32() << endl;
+            kDebug() << "Client_file message: " << msg->readInt32() << " \"" << msg->readString() << "\" " << msg->readInt32();
         } break;
         */
         case DownloadFiles_v1:
         case DownloadFiles_v2:
         case DownloadFiles_v3:
-            kdDebug() << "Obsolete download files message received" << endl;
+            kDebug() << "Obsolete download files message received";
             break;
         case DownloadFiles_v4:
         case DownloadFiles:
@@ -451,7 +461,7 @@ void DonkeyProtocol::processMessage()
             }
         } break;
         case DownloadedFiles_v1:
-            kdDebug() << "Obsolete downloaded files message received" << endl;
+            kDebug() << "Obsolete downloaded files message received";
             break;
         case DownloadedFiles_v2:
         case DownloadedFiles:
@@ -470,7 +480,7 @@ void DonkeyProtocol::processMessage()
         } break;
         case File_info_v1:
         case File_info_v2:
-            kdDebug() << "Obsolete file info message received" << endl;
+            kDebug() << "Obsolete file info message received";
             break;
         case File_info_v3:
         case File_info:
@@ -515,6 +525,7 @@ void DonkeyProtocol::processMessage()
         {
             ServerInfo* si = new ServerInfo(msg, proto);
             ServerInfo* oldsi = findServerNo(si->serverNo());
+
             ClientInfo::State newstate = si->serverState(), oldstate = oldsi ? oldsi->serverState() : ClientInfo::NotConnected;
             if ((oldstate == ClientInfo::Connected || oldstate == ClientInfo::Connected2 || oldstate == ClientInfo::Connected3)
                 && (newstate != ClientInfo::Connected && newstate != ClientInfo::Connected2 && newstate != ClientInfo::Connected3))
@@ -538,7 +549,7 @@ void DonkeyProtocol::processMessage()
             int sn = msg->readInt32();
             ServerInfo* si = findServerNo(sn);
             if (!si) {
-                kdDebug() << "Core sent an invalid server number!" << endl;
+                kDebug() << "Core sent an invalid server number!";
                 break;
             }
             ClientInfo::State oldstate = si->serverState();
@@ -590,10 +601,13 @@ void DonkeyProtocol::processMessage()
         case Options_info:
         {
             int i, j = msg->readInt16();
+            bool ok = true;
             for (i=0; i<j; i++) {
                 QString key, value;
-                key = msg->readString();
-                value = msg->readString();
+                key = msg->readString(&ok);
+                if (! ok) break;
+                value = msg->readString(&ok);
+                if (! ok) break;
                 options.replace(key,value);
             }
             emit optionsUpdated();
@@ -619,7 +633,7 @@ void DonkeyProtocol::processMessage()
         // Search_result's.
         case Result_info:
         {
-            ResultInfo *si = new ResultInfo(msg);
+            ResultInfo *si = new ResultInfo(msg, proto);
             unmappedResults.replace(si->resultNo(), si);
         } break;
 
@@ -647,7 +661,7 @@ void DonkeyProtocol::processMessage()
                 int v = msg->readInt32();
                 o += QString::number(v) + " ";
             }
-            kdDebug() << o << endl;
+            kDebug() << o;
         } break;
         */
 
@@ -657,7 +671,7 @@ void DonkeyProtocol::processMessage()
         case Search:
         {
             int searchnum = msg->readInt32();
-            //kdDebug() << "Search num=" << searchnum << endl;
+            //kDebug() << "Search num=" << searchnum;
             SearchInfo *si = searches[searchnum];
             if (! si) {
                 si = new SearchInfo(searchnum);
@@ -678,26 +692,26 @@ void DonkeyProtocol::processMessage()
 
         case CleanTables:
         {
-            QIntDict<ClientInfo> newClients;
+            Q3IntDict<ClientInfo> newClients;
             int i, j = msg->readInt16();
             for (i=0; i<j; i++) {
                 ClientInfo* cl = clients.take(msg->readInt32());
                 if (cl) newClients.replace(cl->clientNo(), cl);
             }
-            QIntDictIterator<ClientInfo> cit(clients);
+            Q3IntDictIterator<ClientInfo> cit(clients);
             for (; cit.current(); ++cit) {
                 emit clientRemoved(cit.current()->clientNo());
                 pruneClientRecord(cit.current()->clientNo());
             }
             clients = newClients;
 
-            QIntDict<ServerInfo> newServers;
+            Q3IntDict<ServerInfo> newServers;
             j = msg->readInt16();
             for (i=0; i<j; i++) {
                 ServerInfo* si = servers.take(msg->readInt32());
                 if (si) newServers.replace(si->serverNo(), si);
             }
-            QIntDictIterator<ServerInfo> sit(servers);
+            Q3IntDictIterator<ServerInfo> sit(servers);
             for (; sit.current(); ++sit)
                 emit serverRemoved(sit.current()->serverNo());
             servers = newServers;
@@ -752,7 +766,7 @@ void DonkeyProtocol::processMessage()
                 ri->addMessage(rm);
                 emit roomMessage(roomno, rm);
             }
-            else kdDebug() << QString("Room_message for not existing room %1").arg(roomno) << endl;
+            else kDebug() << QString("Room_message for not existing room %1").arg(roomno);
         } break;
 
         case Room_add_user:
@@ -760,7 +774,7 @@ void DonkeyProtocol::processMessage()
             int roomnum = msg->readInt32();
             int usernum = msg->readInt32();
             emit roomAddUser(roomnum, usernum);
-            kdDebug() << "Room_add_user roomnum=" << roomnum << " usernum=" << usernum << endl;
+            kDebug() << "Room_add_user roomnum=" << roomnum << " usernum=" << usernum;
         } break;
 
         case Room_remove_user:
@@ -768,13 +782,13 @@ void DonkeyProtocol::processMessage()
             int roomnum = msg->readInt32();
             int usernum = msg->readInt32();
             emit roomRemoveUser(roomnum, usernum);
-            kdDebug() << "Room_remove_user roomnum=" << roomnum << " usernum=" << usernum << endl;
+            kDebug() << "Room_remove_user roomnum=" << roomnum << " usernum=" << usernum;
         } break;
 
         case UploadFiles:
         {
             for (QMap<int,bool>::Iterator it = upload.begin(); it != upload.end(); ++it)
-                it.data() = false; // We use the bool-value to indicate if an uploaditem gots dirty (removed).
+                it.value() = false; // We use the bool-value to indicate if an uploaditem gots dirty (removed).
             int i, j = msg->readInt16();
             for (i=0; i<j; i++) {
                 int num = msg->readInt32();
@@ -782,7 +796,14 @@ void DonkeyProtocol::processMessage()
                 refreshClientInfo(num);
             }
             for (QMap<int,bool>::Iterator it = upload.begin(); it != upload.end(); ++it)
-                if (! it.data()) refreshClientInfo(it.key());
+                if (! it.value()) refreshClientInfo(it.key());
+        } break;
+
+        case Version:
+        {
+            QString version = msg->readString();
+            emit coreVersion(version);
+            kDebug() << "Core reported version " << version;
         } break;
 
         default:
@@ -808,37 +829,37 @@ void DonkeyProtocol::updateConnectedServers()
     sock.sendMessage(DonkeyMessage(GetConnectedServers));
 }
 
-const QIntDict<FileInfo>& DonkeyProtocol::downloadFiles()
+const Q3IntDict<FileInfo>& DonkeyProtocol::downloadFiles()
 {
     return download;
 }
 
-const QIntDict<FileInfo>& DonkeyProtocol::downloadedFiles()
+const Q3IntDict<FileInfo>& DonkeyProtocol::downloadedFiles()
 {
     return downloaded;
 }
 
-const QIntDict<ServerInfo>& DonkeyProtocol::connectedServers()
+const Q3IntDict<ServerInfo>& DonkeyProtocol::connectedServers()
 {
     return servers;
 }
 
-const QIntDict<Network>& DonkeyProtocol::availableNetworks()
+const Q3IntDict<Network>& DonkeyProtocol::availableNetworks()
 {
     return networks;
 }
 
-const QIntDict<ClientInfo>& DonkeyProtocol::clientList()
+const Q3IntDict<ClientInfo>& DonkeyProtocol::clientList()
 {
     return clients;
 }
 
-const QIntDict<ShareInfo>& DonkeyProtocol::sharedFiles()
+const Q3IntDict<ShareInfo>& DonkeyProtocol::sharedFiles()
 {
     return shares;
 }
 
-const QValueList<int>& DonkeyProtocol::friendList()
+const Q3ValueList<int>& DonkeyProtocol::friendList()
 {
     return friends;
 }
@@ -848,20 +869,6 @@ FileInfo* DonkeyProtocol::findDownloadFileNo(int fileno)
     return download[fileno];
 }
 
-FileInfo* DonkeyProtocol::findDownloadFileByHash(const QByteArray& hash)
-{
-    for (QIntDictIterator<FileInfo> it(download); it.current(); ++it) {
-        if (it.current()->fileMD4() == hash) return it;
-    }
-    return NULL;
-}
-
-FileInfo* DonkeyProtocol::findDownloadFileByHash(const QString& hash)
-{
-    QByteArray foo = FileInfo::stringToMd4(hash);
-    return findDownloadFileByHash(foo);
-}
-
 FileInfo* DonkeyProtocol::findDownloadedFileNo(int fileno)
 {
     return downloaded[fileno];
@@ -954,11 +961,11 @@ void DonkeyProtocol::retryFile(int fileno)
     sock.sendMessage(out);
 }
 
-void DonkeyProtocol::submitURL(const QString& url)
+void DonkeyProtocol::submitUrl(const QString& url)
 {
     downloadstarted = true;
     QRegExp rx("^(ftp|http)://.+");
-    if (rx.search(url) >= 0 && ! url.lower().endsWith(".torrent")) {
+    if (rx.search(url) >= 0 && ! url.toLower().endsWith(".torrent")) {
         sendConsoleMessage("http \"" + url + "\"");
     }
     else {
@@ -1039,7 +1046,7 @@ SearchInfo* DonkeyProtocol::findSearchNo(int num)
     return searches[num];
 }
 
-const QIntDict<SearchInfo>& DonkeyProtocol::activeSearches()
+const Q3IntDict<SearchInfo>& DonkeyProtocol::activeSearches()
 {
     return searches;
 }
@@ -1204,7 +1211,7 @@ void DonkeyProtocol::setOptions(const QMap<QString, QString>& opts)
     out.writeInt16( (int16) opts.count() );
     for (QMap<QString, QString>::ConstIterator it = opts.begin(); it != opts.end(); ++it) {
         out.writeString( it.key() );
-        out.writeString( it.data() );
+        out.writeString( it.value() );
     }
     sock.sendMessage(out);
 }
@@ -1219,12 +1226,12 @@ void DonkeyProtocol::enableNetwork(int nwno, bool enable)
     sock.sendMessage(out);
 }
 
-const QValueList<DonkeyOption>& DonkeyProtocol::sectionOptions()
+const Q3ValueList<DonkeyOption>& DonkeyProtocol::sectionOptions()
 {
     return m_sectionOptions;
 }
 
-const QValueList<DonkeyOption>& DonkeyProtocol::pluginOptions()
+const Q3ValueList<DonkeyOption>& DonkeyProtocol::pluginOptions()
 {
     return m_pluginOptions;
 }
@@ -1242,7 +1249,7 @@ void DonkeyProtocol::sendPrivateMessage(int client, const QString& message)
     sock.sendMessage(out);
 }
 
-const int DonkeyProtocol::coreProtocolVersion()
+const int& DonkeyProtocol::coreProtocolVersion()
 {
     return coreProto;
 }
@@ -1254,7 +1261,7 @@ RoomInfo* DonkeyProtocol::findRoomNo(int roomno)
 
 void DonkeyProtocol::setRoomState(int roomno, RoomInfo::RoomState state)
 {
-    kdDebug() << "DonkeyProtocol::setRoomState() roomno=" << roomno << " state=" << state << endl;
+    kDebug() << "DonkeyProtocol::setRoomState() roomno=" << roomno << " state=" << state;
     DonkeyMessage out(SetRoomState);
     out.writeInt32(roomno);
     out.writeInt32(state);
@@ -1297,6 +1304,36 @@ void DonkeyProtocol::sendNetworkMessage(int network, const QString& msg)
     sock.sendMessage(out);
 }
 
+void DonkeyProtocol::interestedInSources(bool interested)
+{
+    DonkeyMessage out(InterestedInSources);
+    out.writeBool(interested);
+    sock.sendMessage(out);
+}
+
+void DonkeyProtocol::requestVersion()
+{
+    DonkeyMessage out(GetVersion);
+    sock.sendMessage(out);
+}
+
+void DonkeyProtocol::renameServer(int id, const QString& name)
+{
+    DonkeyMessage out(ServerRename);
+    out.writeInt32(id);
+    out.writeString(name);
+    sock.sendMessage(out);
+}
+
+void DonkeyProtocol::setServerPreferred(int id, bool preferred)
+{
+    DonkeyMessage out(ServerSetPreferred);
+    out.writeInt32(id);
+    out.writeBool(preferred);
+    sock.sendMessage(out);
+}
+
+
 
 
 #include "donkeyprotocol.moc"
