@@ -2,8 +2,9 @@
  *
  * donkeyprotocol.h
  *
- * Copyright (C) 2003 Petter E. Stokke <gibreel@kmldonkey.org>
+ * Copyright (C) 2003 Petter Stokke <ummo@hellokitty.com>
  * Copyright (C) 2003 Sebastian Sauer <mail@dipe.org>
+ * Copyright (C) 2006 Christian Muehlhaeuser <chris@chris.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,24 +18,29 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
 #ifndef __libkmldonkey_donkeyprotocol_h__
 #define __libkmldonkey_donkeyprotocol_h__
 
-#include <qintdict.h>
+#include <q3intdict.h>
 #include <qstring.h>
 #include <qmap.h>
+#include <qobject.h>
+//Added by qt3to4:
+#include <Q3ValueList>
 
 #include "donkeysocket.h"
 #include "protocoliface.h"
 #include "options.h"
 #include "roominfo.h"
 
+#include <kmldonkey_export.h>
+
 // Maximum version of GUI protocol supported
-#define MAX_PROTOCOL_VERSION 26
+#define MAX_PROTOCOL_VERSION 33
 
 // Minimum version of GUI protocol supported
 #define MIN_PROTOCOL_VERSION 25
@@ -68,14 +74,14 @@ class SearchQuery;
  * without having to deal with the protocol itself.
  */
 
-class DonkeyProtocol : public ProtocolInterface
+class KMLDONKEY_EXPORT DonkeyProtocol : public ProtocolInterface
 
 {
     Q_OBJECT
 
 public:
 
-    //! Opcodes for core messages.
+    //! Opcodes from core messages.
     enum FromCore {
         CoreProtocol = 0,
         Options_info,
@@ -132,11 +138,76 @@ public:
         DownloadFiles,
         DownloadedFiles,
         UploadFiles,
-        Pending, // from here
-        Search
+        Pending,
+        Search,
+        Version // this is new
+
+	/*TODO: update to last GUIprotocol (v41 ) from core version
+	CoreProtocol,
+	Options_info,
+
+	DefineSearches,
+	Result_info,
+	Search_result,
+	Search_waiting,
+
+	File_info,
+	File_downloaded,
+	File_add_source,
+	File_update_availability,
+	File_remove_source,
+
+	Server_busy,
+	Server_user,
+	Server_state,
+	Server_info,
+
+	Client_info,
+	Client_state,
+	Client_friend,
+	Client_file,
+
+	Console,
+
+	Network_info,
+	User_info,
+
+	Room_info,
+	Room_message,
+	Room_add_user,
+
+	Client_stats,
+
+	ConnectedServers,
+	DownloadFiles,
+	DownloadedFiles,
+	MessageFromClient,
+
+	Room_remove_user,
+	Shared_file_info,
+	Shared_file_upload,
+	Shared_file_unshared,
+
+	Add_section_option,
+
+	Add_plugin_option,
+
+	BadPassword,
+
+	CleanTables,
+	Uploaders,
+	Pending,
+	Search,
+	Version,
+
+	GiftServerAttach,
+	GiftServerStats,
+	Stats,
+
+	*/
     };
 
-    //! Opcodes for GUI messages.
+    //! Opcodes from GUI messages.
     enum ToCore {
         GuiProtocol = 0,
         ConnectMore_query,
@@ -201,7 +272,89 @@ public:
         GetSearch, // 60
         ConnectClient,
         DisconnectClient,
-        NetworkMessage
+        NetworkMessage,
+        InterestedInSources,
+        GetVersion, // this is new
+        ServerRename,
+        ServerSetPreferred
+
+	/*TODO: update to last GUIprotocol(v41) from GUI version
+	GuiProtocol _ -> "GuiProtocol"
+	ConnectMore_query -> "ConnectMore_query"
+	CleanOldServers -> "CleanOldServers"
+	KillServer -> "KillServer"
+	ExtendedSearch _ -> "ExtendedSearch"
+	Password _ -> "Password"
+	Search_query _ -> "Search_query"
+	Download_query _ -> "Download_query"
+	Url _ -> "Url"
+	RemoveServer_query _ -> "RemoveServer_query"
+	SaveOptions_query _ -> "SaveOptions_query"
+	RemoveDownload_query _ -> "RemoveDownload_query"
+	ServerUsers_query _ -> "ServerUsers_query"
+	SaveFile _ -> "SaveFile"
+	AddClientFriend _ -> "AddClientFriend"
+	AddUserFriend _ -> "AddUserFriend"
+	RemoveFriend _ -> "RemoveFriend"
+	RemoveAllFriends -> "RemoveAllFriends"
+	FindFriend _ -> "FindFriend"
+	ViewUsers _ -> "ViewUsers"
+	ConnectAll _ -> "ConnectAll"
+	ConnectServer _ -> "ConnectServer"
+	DisconnectServer _ -> "DisconnectServer"
+	SwitchDownload _ -> "SwitchDownload"
+	VerifyAllChunks _ -> "VerifyAllChunks"
+	QueryFormat _ -> "QueryFormat"
+	ModifyMp3Tags _ -> "ModifyMp"
+	CloseSearch _ -> "CloseSearch"
+	SetOption _ -> "SetOption"
+	Command _ -> "Command"
+	Preview _ -> "Preview"
+	ConnectFriend _ -> "ConnectFriend"
+	GetServer_users _ -> "GetServer_users"
+	GetClient_files _ -> "GetClient_files"
+	GetFile_locations _ -> "GetFile_locations"
+	GetServer_info _ -> "GetServer_info"
+	GetClient_info _ -> "GetClient_info"
+	GetFile_info _ -> "GetFile_info"
+	GetUser_info n -> Printf.sprintf "GetUser_info %d" n
+	SendMessage _ -> "SendMessage"
+	EnableNetwork _ -> "EnableNetwork"
+	BrowseUser _ -> "BrowseUser"
+
+	MessageToClient _ -> "MessageToClient"
+	GuiExtensions _ -> "GuiExtensions"
+	GetConnectedServers -> "GetConnectedServers"
+	GetDownloadFiles -> "GetDownloadFiles"
+	GetDownloadedFiles -> "GetDownloadedFiles"
+	SetRoomState _ -> "CloseRoom"
+
+	RefreshUploadStats -> "RefreshUploadStats"
+	SetFilePriority _ -> "SetFilePriority"      
+	AddServer_query _ -> "AddServer_query"
+
+	MessageVersions _ -> "MessageVersions"
+
+	RenameFile _ -> "RenameFile"
+	GetUploaders -> "GetUploaders"
+	GetPending -> "GetPending"
+	GetSearches -> "GetSearches"
+	GetSearch _ -> "GetSearch"
+
+	ConnectClient _ -> "ConnectClient"
+	DisconnectClient _ -> "DisconnectClient"
+	NetworkMessage _ -> "NetworkMessage"
+
+	GiftAttach _ -> "GiftAttach"
+	GiftStats -> "GiftStats"
+
+	InterestedInSources _ -> "InterestedInSources"
+	GetVersion -> "GetVersion"
+
+	ServerRename _ -> "ServerRename"
+	ServerSetPreferred _ -> "ServerSetPreferred"
+	GetStats _ -> "GetStats"
+	*/
     };
 
     //! Search types.
@@ -290,7 +443,7 @@ public:
      * \return The current protocol version.
      * \sa coreProtocolVersion()
      */
-    const int protocolVersion();
+    const int& protocolVersion();
 
     //! Get the core's protocol version.
     /*!
@@ -300,56 +453,56 @@ public:
      * \return The core's protocol version.
      * \sa protocolVersion()
      */
-    const int coreProtocolVersion();
+    const int& coreProtocolVersion();
 
     //! Get the currently downloading files.
     /*!
      * \return A dictionary containing FileInfo objects representing the
      *   currently downloading files, indexed on their fileNo() values.
      */
-    const QIntDict<FileInfo>& downloadFiles();
+    const Q3IntDict<FileInfo>& downloadFiles();
 
     //! Get the completed files.
     /*!
      * \return A dictionary containing FileInfo objects representing the
      *   completed files, indexed on their fileNo() values.
      */
-    const QIntDict<FileInfo>& downloadedFiles();
+    const Q3IntDict<FileInfo>& downloadedFiles();
 
     //! Get the connected servers.
     /*!
      * \return A dictionary containing ServerInfo objects representing the
      *   currently connected servers, indexed on their serverNo() values.
      */
-    const QIntDict<ServerInfo>& connectedServers();
+    const Q3IntDict<ServerInfo>& connectedServers();
 
     //! Get the network list.
     /*!
      * \return A dictionary containing Network objects representing the
      *   available networks, indexed on their networkNo() values.
      */
-    const QIntDict<Network>& availableNetworks();
+    const Q3IntDict<Network>& availableNetworks();
 
     //! Get the client list.
     /*!
      * \return A dictionary containing ClientInfo objects representing
      *   the clients, indexed on their clientNo() values.
      */
-    const QIntDict<ClientInfo>& clientList();
+    const Q3IntDict<ClientInfo>& clientList();
 
     //! Get the shared file list.
     /*!
      * \return A dictionary containing ShareInfo objects representing
      *   the shares, indexed on their shareNo() values.
      */
-    const QIntDict<ShareInfo>& sharedFiles();
+    const Q3IntDict<ShareInfo>& sharedFiles();
 
     //! Get the list of active searches.
     /*!
      * \return A dictionary containing SearchInfo objects representing
      *   the active searches, indexed on their searchNo() values.
      */
-    const QIntDict<SearchInfo>& activeSearches();
+    const Q3IntDict<SearchInfo>& activeSearches();
 
     //! Find a downloading file by its file number.
     /*!
@@ -360,26 +513,6 @@ public:
      */
     FileInfo* findDownloadFileNo(int fileno);
 
-    //! Find a downloading file by its MD4 hash.
-    /*!
-     * \param hash The hash to search for.
-     * \return A pointer to the FileInfo object describing the requested
-     *   hash, or NULL if there is no file being downloaded with the given
-     *   hash.
-     * \sa downloadFiles(), findDownloadFileNo(), findDownloadFileByHash(QString&)
-     */
-    FileInfo* findDownloadFileByHash(const QByteArray& hash);
-
-    //! Find a downloading file by its MD4 hash string representation.
-    /*!
-     * \param hash String representation of the hash to search for.
-     * \return A pointer to the FileInfo object describing the requested
-     *   hash, or NULL if there is no file being downloaded with the given
-     *   hash.
-     * \sa downloadFiles(), findDownloadFileNo(), findDownloadFileByHash(QByteArray&)
-     */
-    FileInfo* findDownloadFileByHash(const QString& hash);
-
     //! Find a downloaded file by its file number.
     /*!
      * \param fileno The file number to search for.
@@ -511,7 +644,7 @@ public:
      * Submit an ed2k URL to the core for downloading.
      * \param url The ed2k URL to download.
      */
-    void submitURL(const QString& url);
+    void submitUrl(const QString& url);
 
     //! Send a DonkeyMessage to the core.
     /*!
@@ -637,7 +770,7 @@ public:
     //! Get a list of friend clients.
     /*! \return A list of ints corresponding to the client IDs of the current friends.
      */
-    const QValueList<int>& friendList();
+    const Q3ValueList<int>& friendList();
     //! Search for a friend.
     /*! \param name The name of the friend to search for.
      */
@@ -691,13 +824,13 @@ public:
      * Return a list of section options.
      * \return A list of section options, defined by DonkeyOption objects
      */
-    const QValueList<DonkeyOption>& sectionOptions();
+    const Q3ValueList<DonkeyOption>& sectionOptions();
 
     /**
      * Return a list of plugin options.
      * \return A list of plugin options, defined by DonkeyOption objects
      */
-    const QValueList<DonkeyOption>& pluginOptions();
+    const Q3ValueList<DonkeyOption>& pluginOptions();
 
     /**
      * Kill the MLDonkey core.
@@ -758,6 +891,17 @@ public:
      */
     void sendNetworkMessage(int network, const QString& msg);
 
+    /**
+     * Tell the core whether we're interested in learning about file sources.
+     * @param interested If false, messages about sources will not be sent by the core.
+     */
+    void interestedInSources(bool interested);
+
+    /**
+     * Request the core's version.
+     */
+    void requestVersion();
+
 public slots:
 
     //! Request a new download file list.
@@ -798,6 +942,20 @@ public slots:
      */
     void sendConsoleMessage(const QString& msg, ConsoleCallbackInterface* callback = 0);
 
+    /**
+     * Rename a server.
+     * @param id The server ID.
+     * @param name The server's new name.
+     */
+    void renameServer(int id, const QString& name);
+
+    /**
+     * Change a server's preferred flag.
+     * @param id The server ID.
+     * @param preferred The server's new preferred flag.
+     */
+    void setServerPreferred(int id, bool preferred);
+
 protected slots:
 
     //! Process any messages in the DonkeySocket queue.
@@ -813,18 +971,18 @@ private:
 
     DonkeySocket sock;
     QString uname, passwd;
-    QIntDict<FileInfo> download;
-    QIntDict<FileInfo> downloaded;
+    Q3IntDict<FileInfo> download;
+    Q3IntDict<FileInfo> downloaded;
     QMap<int,bool> upload;
-    QIntDict<ServerInfo> servers;
-    QIntDict<Network> networks;
-    QIntDict<ClientInfo> clients;
-    QIntDict<ShareInfo> shares;
-    QIntDict<RoomInfo> rooms;
-    QIntDict<SearchInfo> searches;
-    QIntDict<ResultInfo> unmappedResults;
-    QValueList<DonkeyOption> m_sectionOptions;
-    QValueList<DonkeyOption> m_pluginOptions;
+    Q3IntDict<ServerInfo> servers;
+    Q3IntDict<Network> networks;
+    Q3IntDict<ClientInfo> clients;
+    Q3IntDict<ShareInfo> shares;
+    Q3IntDict<RoomInfo> rooms;
+    Q3IntDict<SearchInfo> searches;
+    Q3IntDict<ResultInfo> unmappedResults;
+    Q3ValueList<DonkeyOption> m_sectionOptions;
+    Q3ValueList<DonkeyOption> m_pluginOptions;
     QMap<QString,QString> options;
     QMap<QString,ConsoleCallbackInterface*> consoleCallbacks;
     QMap<int,int> clientstatsmap;
@@ -833,7 +991,7 @@ private:
     int proto, coreProto;
     uint connectedservers;
     bool wantpoll, downloadstarted;
-    QValueList<int> friends;
+    Q3ValueList<int> friends;
 
     QString defSearch;
     QMap<QString, SearchQuery*> defSearches;
@@ -1043,6 +1201,13 @@ signals:
      * @param searchid The ID of the search.
      */
     void searchRequest(int searchid);
+
+    /**
+     * A core version string has arrived.
+     * @param versionString The core's version string.
+     */
+    void coreVersion(const QString& versionString);
+
 };
 
 #endif
